@@ -82,6 +82,86 @@ summary.modsem_da <- function(object,
                               ...) {
   method   <- object$method
   parTable <- parameter_estimates(object)
+  finalModel <- object$model
+  groupModels <- finalModel$models
+
+  if (is.null(groupModels) || !length(groupModels))
+    groupModels <- list(list(data = object$data))
+
+  n.groups <- length(groupModels)
+  group.labels <- finalModel$info$group.levels
+  if (is.null(group.labels) || length(group.labels) != n.groups) {
+    model.names <- names(groupModels)
+    if (!is.null(model.names) && length(model.names) == n.groups &&
+        all(nzchar(model.names))) {
+      group.labels <- model.names
+    } else {
+      group.labels <- paste0("Group ", seq_len(n.groups))
+    }
+  }
+
+  normalize_r2 <- function(x) {
+    if (is.null(x)) return(NULL)
+    vec <- unclass(x)
+    nm  <- names(vec)
+    vec <- as.numeric(vec)
+    names(vec) <- nm
+    vec
+  }
+
+  aggregate_r2 <- function(r2, weights) {
+    if (is.null(r2)) return(NULL)
+    if (!is.list(r2)) return(normalize_r2(r2))
+    if (!length(r2)) return(NULL)
+    r2_names <- names(r2)
+    if (!is.null(names(weights)) &&
+        !is.null(r2_names) &&
+        length(r2_names) == length(r2) &&
+        all(r2_names %in% names(weights))) {
+      weights <- weights[r2_names]
+    } else {
+      weights <- weights[seq_len(length(r2))]
+    }
+    vars <- unique(unlist(lapply(r2, names)))
+    vars <- vars[!is.na(vars) & nzchar(vars)]
+    if (!length(vars)) return(NULL)
+    agg <- stats::setNames(numeric(length(vars)), vars)
+    total_w <- sum(weights, na.rm = TRUE)
+    if (!is.finite(total_w) || total_w <= 0) {
+      total_w <- length(r2)
+      weights[] <- 1
+    }
+    for (i in seq_along(r2)) {
+      vec <- normalize_r2(r2[[i]])
+      if (is.null(vec) || !length(vec)) next
+      w <- weights[[i]]
+      if (!is.finite(w) || w < 0) w <- 0
+      agg[names(vec)] <- agg[names(vec)] + vec * w
+    }
+    agg / total_w
+  }
+
+  extract_n <- function(dat) {
+    if (is.null(dat)) return(0)
+    if (!is.null(dat$n)) return(dat$n)
+    if (!is.null(dat$data.full)) return(NROW(dat$data.full))
+    if (!is.null(dat$data)) return(NROW(dat$data))
+    0
+  }
+
+  extract_patterns <- function(dat) {
+    if (is.null(dat)) return(0L)
+    if (isTRUE(dat$is.fiml)) {
+      if (!is.null(dat$p)) return(dat$p)
+      if (!is.null(dat$ids)) return(length(dat$ids))
+      return(1L)
+    }
+    0L
+  }
+
+  group.Ns <- vapply(groupModels, function(sub) extract_n(sub$data), numeric(1L))
+  fiml.counts <- vapply(groupModels, function(sub) extract_patterns(sub$data), numeric(1L))
+  fiml.flags  <- fiml.counts > 0L
 
   extra.cols <- NULL
   if (standardized) {
@@ -114,27 +194,30 @@ summary.modsem_da <- function(object,
     )
   }
 
-  if (!var.interaction) {
+  if (!var.interaction)
     parTable.out <- removeInteractionVariances(parTable)
-  } else parTable.out <- parTable
+  else
+    parTable.out <- parTable
 
   args <- object$args
   out <- list(
     parTable        = parTable.out,
-    data            = object$data$data.full,
+    data            = NULL,
     iterations      = object$iterations,
     logLik          = object$logLik,
-    fit             = fit_modsem_da(object, chisq = FALSE),
+    fit             = fitModsemDA_Internal(object, chisq = FALSE),
     D               = NULL,
-    N               = NROW(object$data$data.full),
+    N               = structure(group.Ns, names = group.labels),
+    N.total         = sum(group.Ns),
     method          = method,
     optimizer       = object$optimizer,
     quad            = object$info.quad,
     type.se         = object$type.se,
     type.estimates  = ifelse(standardized, "standardized", object$type.estimates),
     information     = object$information,
-    n.fiml.patterns = length(object$data$ids),
-    is.fiml         = object$data$is.fiml,
+    is.fiml         = any(fiml.flags),
+    n.fiml.patterns = structure(fiml.counts, names = group.labels),
+    group.labels    = group.labels,
     npar            = length(coef(object, type = "free")),
     convergence.msg = object$convergence.msg
   )
@@ -154,22 +237,28 @@ summary.modsem_da <- function(object,
 
     } else {
       out$D     <- compare_fit(est_h1 = object, est_h0 = est_h0)
-      out$fitH0 <- fit_modsem_da(est_h0, lav.fit = TRUE)
+      out$fitH0 <- fitModsemDA_Internal(est_h0, lav.fit = TRUE)
     }
   } else {
     out$D <- NULL
   }
 
   if (r.squared) {
-  	out$r.squared <- modsem_inspect(object, "r2.lv")
+    r2.values <- modsem_inspect(object, "r2.lv")
+    out$r.squared <- aggregate_r2(r2.values, group.Ns)
 
     if (H0) {
-			r.squared.h0 <- modsem_inspect(est_h0, "r2.lv")
-			out$r.squared.h0 <- r.squared.h0[names(out$r.squared)] # should't be necessary
-			                                                       # but sort just in case...
-		} else out$r.squared.h0 <- NULL
+      r2.h0.values <- modsem_inspect(est_h0, "r2.lv")
+      out$r.squared.h0 <- aggregate_r2(r2.h0.values, group.Ns)
 
-  } else out$r.squared <- NULL
+    } else {
+      out$r.squared.h0 <- NULL
+    }
+
+  } else {
+    out$r.squared <- NULL
+    out$r.squared.h0 <- NULL
+  }
 
   out$format <- list(
     digits        = digits,
@@ -193,6 +282,21 @@ summary.modsem_da <- function(object,
 
 #' @export
 print.summary_da <- function(x, digits = 3, ...) {
+  format_value <- function(val, digits = 3, scientific = x$format$scientific) {
+    if (length(val) == 0 || all(is.na(val))) {
+      return("NA")
+    }
+    formatNumeric(val, digits = digits, scientific = scientific)
+  }
+
+  formatCount <- function(val) {
+    if (length(val) == 0) return("NA")
+    num <- suppressWarnings(as.numeric(val))
+    out <- ifelse(is.na(num), "NA",
+                  formatC(round(num), format = "f", digits = 0, big.mark = ""))
+    out
+  }
+
   # We want the width without ci and extra cols
   width.out <- getWidthPrintedParTable(
     parTable    = x$parTable,
@@ -210,31 +314,77 @@ print.summary_da <- function(x, digits = 3, ...) {
   printf(x$convergence.msg)
 
   # Convergence and Model Info -------------------------------------------------
-  names <- c(
+  header.names <- c(
     "Estimator",
     "Optimization method",
-    "Number of model parameters",
-    "", # blank line
-    "Number of observations",
-    "Number of missing patterns"
+    "Number of model parameters"
   )
 
-  values <- c(
+  header.values <- c(
     stringr::str_to_upper(c(x$method, x$optimizer)),
-    x$npar,
-    "", # blank line
-    x$N,
-    x$n.fiml.patterns
+    x$npar
   )
 
-  if (!x$is.fiml) {
-    fieldFIML <- grepl("Number of missing patterns", names)
-    names  <- names[!fieldFIML]
-    values <- values[!fieldFIML]
+  cat(allignLhsRhs(lhs = header.names, rhs = header.values, pad = "  ",
+                   width.out = width.out), "\n", sep = "")
+
+  nObsVec <- x$N
+  if (is.null(names(nObsVec)) || !any(nzchar(names(nObsVec))))
+    names(nObsVec) <- paste0("Group ", seq_along(nObsVec))
+
+  total.n <- sum(nObsVec, na.rm = TRUE)
+  sampleLines <- allignLhsRhs(
+    lhs = "Number of observations",
+    rhs = formatCount(total.n),
+    pad = "  ",
+    width.out = width.out
+  )
+
+  if (length(nObsVec) > 1L) {
+    sampleLines <- paste0(
+      sampleLines,
+      allignLhsRhs(
+        lhs = paste0("  ", names(nObsVec)),
+        rhs = formatCount(nObsVec),
+        pad = "  ",
+        width.out = width.out
+      )
+    )
   }
 
-  cat(allignLhsRhs(lhs = names, rhs = values, pad = "  ",
-                   width.out = width.out), "\n")
+  if (x$is.fiml) {
+    fimlVec <- x$n.fiml.patterns
+
+    if (is.null(names(fimlVec)) || !any(nzchar(names(fimlVec))))
+      names(fimlVec) <- paste0("Group ", seq_along(fimlVec))
+
+    totalPatterns <- sum(fimlVec, na.rm = TRUE)
+    sampleLines <- paste0(
+      sampleLines,
+      allignLhsRhs(
+        lhs = "Number of missing patterns",
+        rhs = formatCount(totalPatterns),
+        pad = "  ",
+        width.out = width.out
+      )
+    )
+
+    nonZero <- fimlVec > 0
+    if (length(fimlVec) > 1L && any(nonZero)) {
+      sampleLines <- paste0(
+        sampleLines,
+        allignLhsRhs(
+          lhs = paste0("  ", names(fimlVec)[nonZero]),
+          rhs = formatCount(fimlVec[nonZero]),
+          pad = "  ",
+          width.out = width.out
+        )
+      )
+    }
+
+  }
+
+  cat(sampleLines, "\n", sep = "")
 
   # Criterion/LogLik -----------------------------------------------------------
   names <- c(
@@ -244,15 +394,15 @@ print.summary_da <- function(x, digits = 3, ...) {
   )
 
   values <- c(
-    formatNumeric(x$logLik,  digits = 2),
-    formatNumeric(x$fit$AIC, digits = 2),
-    formatNumeric(x$fit$BIC, digits = 2)
+    format_value(x$logLik,  digits = 2),
+    format_value(x$fit$AIC, digits = 2),
+    format_value(x$fit$BIC, digits = 2)
   )
 
   if (x$format$adjusted.stat) {
     names  <- c(names, "Corrected Akaike (AICc)", "Adjusted Bayesian (aBIC)")
-    values <- c(values, formatNumeric(x$fit$AICc, digits = 2),
-                formatNumeric(x$fit$aBIC, digits = 2))
+    values <- c(values, format_value(x$fit$AICc, digits = 2),
+                format_value(x$fit$aBIC, digits = 2))
   }
 
   cat("Loglikelihood and Information Criteria:\n")
@@ -261,12 +411,20 @@ print.summary_da <- function(x, digits = 3, ...) {
 
   # Intergration ---------------------------------------------------------------
   if (!is.null(x$quad)) {
-    cat("Numerical Integration:\n")
-    names <- c("Points of integration (per dim)", "Dimensions",
-               "Total points of integration")
-    values <- c(x$quad$nodes.dim, x$quad$dim, x$quad$nodes.total)
-    cat(allignLhsRhs(lhs = names, rhs = values, pad = "  ",
-                     width.out = width.out), "\n")
+    nodesDim   <- x$quad$nodes.dim
+    dimVal     <- x$quad$dim
+    nodesTotal <- x$quad$nodes.total
+
+    if (length(nodesDim) || length(dimVal) || length(nodesTotal)) {
+      cat("Numerical Integration:\n")
+      names <- c("Points of integration (per dim)", "Dimensions",
+                 "Total points of integration")
+      values <- c(formatCount(nodesDim),
+                  formatCount(dimVal),
+                  formatCount(nodesTotal))
+      cat(allignLhsRhs(lhs = names, rhs = values, pad = "  ",
+                       width.out = width.out), "\n", sep = "")
+    }
 
   }
 
@@ -280,7 +438,7 @@ print.summary_da <- function(x, digits = 3, ...) {
     names <- c("", "Chi-square", "Degrees of Freedom (Chi-square)",
                "P-value (Chi-square)")
     values <- c("Standard",
-                formatNumeric(x$fitH0$chisq.value, digits = 2),
+                format_value(x$fitH0$chisq.value, digits = 2),
                 x$fitH0$chisq.df,
                 formatPval(x$fitH0$chisq.pvalue, scientific = x$format$scientific))
 
@@ -290,10 +448,10 @@ print.summary_da <- function(x, digits = 3, ...) {
       pval.s  <- fnull(lav.fit.h0[["pvalue.scaled"]])
       scale.f <- fnull(lav.fit.h0[["chisq.scaling.factor"]])
       values.scaled <- c("Scaled",
-                         formatNumeric(chisq.s, digits = 2),
+                         format_value(chisq.s, digits = 2),
                          round(df.s),
                          formatPval(pval.s, scientific = x$format$scientific),
-                         formatNumeric(scale.f, digits = 3),
+                         format_value(scale.f, digits = 3),
                          rep("", 2))
       values <- c(values, rep("", 3))
       names <- c(names, "Scaling correction factor",
@@ -303,11 +461,11 @@ print.summary_da <- function(x, digits = 3, ...) {
 
 
     names <- c(names, "RMSEA")
-    values <- c(values, formatNumeric(x$fitH0$RMSEA, digits = 3))
+    values <- c(values, format_value(x$fitH0$RMSEA, digits = 3))
 
     if (!is.null(values.scaled)) {
       rmsea.s <- fnull(lav.fit.h0[["rmsea.scaled"]])
-      values.scaled <- c(values.scaled, formatNumeric(rmsea.s, digits = 3))
+      values.scaled <- c(values.scaled, format_value(rmsea.s, digits = 3))
     }
 
     if (x$format$extra.fit) {
@@ -316,25 +474,25 @@ print.summary_da <- function(x, digits = 3, ...) {
       cfi  <- fnull(lav.fit.h0[["cfi"]])
       tli  <- fnull(lav.fit.h0[["tli"]])
       values <- c(values,
-                  formatNumeric(cfi, digits = 3),
-                  formatNumeric(tli, digits = 3),
-                  formatNumeric(srmr, digits = 3))
+                  format_value(cfi, digits = 3),
+                  format_value(tli, digits = 3),
+                  format_value(srmr, digits = 3))
 
       if (!is.null(values.scaled)) {
         cfi.s <- fnull(lav.fit.h0[["cfi.scaled"]])
         tli.s <- fnull(lav.fit.h0[["tli.scaled"]])
         values.scaled <- c(values.scaled,
-                           formatNumeric(cfi.s, digits = 3),
-                           formatNumeric(tli.s, digits = 3),
+                           format_value(cfi.s, digits = 3),
+                           format_value(tli.s, digits = 3),
                            "")
       }
     }
 
     names <- c(names, "", "Loglikelihood", "Akaike (AIC)", "Bayesian (BIC)")
     values <- c(values, "",
-                formatNumeric(x$nullModel$logLik, digits = 2),
-                formatNumeric(x$fitH0$AIC, digits = 2),
-                formatNumeric(x$fitH0$BIC, digits = 2))
+                format_value(x$nullModel$logLik, digits = 2),
+                format_value(x$fitH0$AIC, digits = 2),
+                format_value(x$fitH0$BIC, digits = 2))
 
     if (!is.null(values.scaled))
       values.scaled <- c(values.scaled, rep("", 4))
@@ -342,8 +500,8 @@ print.summary_da <- function(x, digits = 3, ...) {
     if (x$format$adjusted.stat) {
       names <- c(names, "Corrected Akaike (AICc)", "Adjusted Bayesian (aBIC)")
       values <- c(values,
-                  formatNumeric(x$fitH0$AICc, digits = 2),
-                  formatNumeric(x$fitH0$aBIC, digits = 2))
+                  format_value(x$fitH0$AICc, digits = 2),
+                  format_value(x$fitH0$aBIC, digits = 2))
 
       if (!is.null(values.scaled))
         values.scaled <- c(values.scaled, rep("", 2))
@@ -356,8 +514,8 @@ print.summary_da <- function(x, digits = 3, ...) {
     names <- c("Loglikelihood change",
                "Difference test (D)",
                "Degrees of freedom (D)", "P-value (D)")
-    values <- c(formatNumeric(x$D$diff.loglik, digits = 2),
-                formatNumeric(x$D$D, digits = 2),
+    values <- c(format_value(x$D$diff.loglik, digits = 2),
+                format_value(x$D$D, digits = 2),
                 x$D$df,
                 formatPval(x$D$p, scientific = x$format$scientific))
     cat(allignLhsRhs(lhs = names, rhs = values, pad = "  ",
@@ -367,27 +525,36 @@ print.summary_da <- function(x, digits = 3, ...) {
 
   # R2 -------------------------------------------------------------------------
   if (!is.null(x$r.squared)) {
-    r.squared <- formatNumeric(x$r.squared, digits = 3)
+    r.squared <- format_value(x$r.squared, digits = 3)
     names     <- names(r.squared)
 
-    cat("R-Squared Interaction Model (H1):\n")
+    label_h1 <- "R-Squared Interaction Model (H1)"
+    if (length(x$N) > 1) label_h1 <- paste0(label_h1, " (weighted)")
+
+    cat(label_h1, ":\n", sep = "")
     cat(allignLhsRhs(lhs = names, rhs = r.squared,
 										 pad = "  ", width.out = width.out))
 
     if (!is.null(x$r.squared.h0)) {
-      r.squared.h0 <- formatNumeric(x$r.squared.h0, digits = 3)
+      r.squared.h0 <- format_value(x$r.squared.h0, digits = 3)
 			names.h0     <- names(r.squared.h0)
 
-			cat("R-Squared Baseline Model (H0):\n")
+      label_h0 <- "R-Squared Baseline Model (H0)"
+      if (length(x$N) > 1) label_h0 <- paste0(label_h0, " (weighted)")
+
+			cat(label_h0, ":\n", sep = "")
       cat(allignLhsRhs(lhs = names.h0, rhs = r.squared.h0, pad = "  ",
                        width.out = width.out))
 
       # Calculate Change (using unformatted Rsquared)
-     	r.squared.diff <- formatNumeric(x$r.squared - x$r.squared.h0, digits = 3)
-			names.diff     <- names(r.squared.diff)
-      cat("R-Squared Change (H1 - H0):\n")
-      cat(allignLhsRhs(lhs = names.diff, rhs = r.squared.diff,
-											 pad = "  ", width.out = width.out))
+      if (!is.null(x$r.squared) && !is.null(x$r.squared.h0)) {
+        r.squared.diff.raw <- x$r.squared - x$r.squared.h0
+        r.squared.diff <- format_value(r.squared.diff.raw, digits = 3)
+        names.diff     <- names(r.squared.diff)
+        cat("R-Squared Change (H1 - H0):\n")
+        cat(allignLhsRhs(lhs = names.diff, rhs = r.squared.diff,
+                         pad = "  ", width.out = width.out))
+      }
     }
   }
 
@@ -398,31 +565,36 @@ print.summary_da <- function(x, digits = 3, ...) {
   cat(allignLhsRhs(lhs = names, rhs = values, pad = "  ",
                    width.out = width.out), "\n")
 
-  printParTable(x$parTable,
-                scientific  = x$format$scientific,
-                ci          = x$format$ci,
-                digits      = x$format$digits,
-                loadings    = x$format$loadings,
-                regressions = x$format$regressions,
-                covariances = x$format$covariances,
-                intercepts  = x$format$intercepts,
-                variances   = x$format$variances,
-                extra.cols  = x$format$extra.cols)
+  groups  <- getGroupsParTable(x$parTable)
+  ngroups <- length(groups)
+
+  for (g in groups) {
+    if (ngroups > 1L) {
+      label <- tryCatch(x$group.labels[[g]], error = \(e) NA)
+      printf("Group %d [%s]:\n\n", g, label)
+    }
+
+    select <- x$parTable$group == g
+    if (g == max(groups)) select <- select | x$parTable$group == 0L
+
+    printParTable(x$parTable[select, , drop = FALSE],
+                  scientific  = x$format$scientific,
+                  ci          = x$format$ci,
+                  digits      = x$format$digits,
+                  loadings    = x$format$loadings,
+                  regressions = x$format$regressions,
+                  covariances = x$format$covariances,
+                  intercepts  = x$format$intercepts,
+                  variances   = x$format$variances,
+                  extra.cols  = x$format$extra.cols)
+  }
 }
 
 
 #' @export
 print.modsem_da <- function(x, digits = 3, ...) {
-  parTable         <- x$parTable
-  parTable$p.value <- format.pval(parTable$p.value, digits = digits)
-  names(parTable)  <- c("lhs", "op", "rhs", "label", "est", "std.error",
-                        "z.value", "p.value", "ci.lower", "ci.upper")
-  est <- lapply(parTable, FUN = function(col)
-                if (is.numeric(col)) round(col, digits) else col) |>
-    as.data.frame()
-
   cat(x$convergence.msg)
-  print(est)
+  print(parameter_estimates(x))
 }
 
 
@@ -466,6 +638,10 @@ var_interactions.modsem_da <- function(object, ...) {
 #'
 #' \describe{
 #' \item{\code{"N"}}{Number of analysed rows (integer).}
+#' \item{\code{"ngroups"}}{Number of groups in model (integer).}
+#' \item{\code{"group"}}{Group variable in model (character).}
+#' \item{\code{"group.label"}}{Group labels (character).}
+#' \item{\code{"ovs"}}{Observed variables used in model (character).}
 #' }
 #'
 #' \strong{Parameter estimates and standard errors:}
@@ -683,41 +859,76 @@ modsem_predict.modsem_da <- function(object, standardized = FALSE, H0 = TRUE, ne
     if (is.null(modelH0)) modelH0 <- modelH1
   } else modelH0 <- modelH1
 
-  if (!is.null(newdata)) {
-    cols <- colnames(modelH0$data$data.full)
-    cols.present <- cols %in% colnames(newdata)
-    stopif(!all(cols.present), "Missing cols in `newdata`:\n", cols[!cols.present])
-
-    newdata <- as.matrix(newdata)[, cols]
-
-  } else newdata <- modelH0$data$data.full
-
   transform.x <- if (center.data) \(x) x - mean(x, na.rm = TRUE) else \(x) x
 
   parTableH1 <- parameter_estimates(modelH1)
   parTableH0 <- parameter_estimates(modelH0)
 
-  lVs   <- getLVs(parTableH1)
-  sigma <- modsem_inspect(modelH0, what = "cov.ov")
+  parTableH1 <- addMissingGroups(parTableH1)
+  parTableH0 <- addMissingGroups(parTableH0)
 
-  sigma.inv <- GINV(sigma)
-  lambda    <- getLambdaParTable(parTableH0, rows = colnames(sigma), cols = lVs)
+  groups <- getGroupsParTable(parTableH0)
+  mgroup <- length(groups) > 1L
 
-  X <- apply(as.matrix(newdata), MARGIN = 2, FUN = transform.x)
-  X <- X[, colnames(sigma), drop = FALSE]
+  if (!is.null(newdata)) {
+    newdata <- as.data.frame(newdata)
+    cols  <- colnames(modelH0$model$models[[1L]]$data$data.full)
 
-  FSC <- GINV(t(lambda) %*% sigma.inv %*% lambda) %*% (t(lambda) %*% sigma.inv)
+    cols.present <- cols %in% colnames(newdata)
+    stopif(!all(cols.present), "Missing cols in `newdata`:\n", cols[!cols.present])
 
-  alpha <- matrix(getMeans(lVs, parTable = parTableH1),
-                  nrow = nrow(X), ncol = length(lVs), byrow = TRUE)
+    group <- modsem_inspect(modelH0, what = "group")
 
-  Y <- X %*% t(FSC) + alpha
+    if (!is.null(group)) {
+      group.values <- as.character(newdata[[group]])
+      group.levels <- modsem_inspect(modelH0, what = "group.label")
 
-  if (standardized) {
-    mu <- \(x) mean(x, na.rm = TRUE)
-    s  <- \(x) stats::sd(x, na.rm = TRUE)
-    Y  <- apply(Y, MARGIN = 2, FUN = \(y) (y - mu(y)) / s(y))
+      NEWDATA <- lapply(
+        X = group.levels,
+        FUN = \(g) as.matrix(newdata[newdata[[group]] == g, , drop = FALSE])[, cols]
+      )
+
+    } else {
+      NEWDATA <- list(as.matrix(newdata)[, cols])
+    }
+
+  } else {
+    NEWDATA <- lapply(modelH0$model$models,
+                      FUN = \(sub) sub$data$data.full)
   }
 
-  Y
+  SIGMA <- modsem_inspect(modelH0, what = "cov.ov")
+  out <- vector("list", length = length(groups))
+
+  for (g in groups) {
+    parTableH1g <- parTableH1[parTableH1$group == g, , drop = FALSE]
+    parTableH0g <- parTableH0[parTableH0$group == g, , drop = FALSE]
+
+    lVs   <- getLVs(parTableH1g)
+    sigma <- if (mgroup) SIGMA[[g]] else SIGMA
+
+    sigma.inv <- GINV(sigma)
+    lambda    <- getLambdaParTable(parTableH0g, rows = colnames(sigma), cols = lVs)
+
+    newdata.g <- NEWDATA[[g]]
+    X <- apply(as.matrix(newdata.g), MARGIN = 2, FUN = transform.x)
+    X <- X[, colnames(sigma), drop = FALSE]
+
+    FSC <- GINV(t(lambda) %*% sigma.inv %*% lambda) %*% (t(lambda) %*% sigma.inv)
+
+    alpha <- matrix(getMeans(lVs, parTable = parTableH1g),
+                    nrow = nrow(X), ncol = length(lVs), byrow = TRUE)
+
+    Y <- X %*% t(FSC) + alpha
+
+    if (standardized) {
+      mu <- \(x) mean(x, na.rm = TRUE)
+      s  <- \(x) stats::sd(x, na.rm = TRUE)
+      Y  <- apply(Y, MARGIN = 2, FUN = \(y) (y - mu(y)) / s(y))
+    }
+
+    out[[g]] <- Y
+  }
+
+  if (mgroup) out else out[[1L]]
 }
