@@ -4,6 +4,9 @@
 #'
 #' @param data A dataframe with observed variables used in the model.
 #'
+#' @param group Character. A variable name in the data frame defining the groups in a multiple
+#'   group analysis
+#'
 #' @param method method to use:
 #' \describe{
 #'   \item{\code{"lms"}}{latent moderated structural equations (not passed to \code{lavaan}).}
@@ -141,8 +144,9 @@
 #'
 #' @param em.control a list of control parameters for the EM algorithm. See \code{\link{default_settings_da}} for defaults.
 #'
-#' @param ordered Variables to be treated as ordered. The scale of the ordinal variables
-#'   is scaled to correct for unequal intervals. The underlying continous distributions
+#' @param ordered Variables to be treated as ordered. Categories for ordered variables
+#'   are scored, transforming them from ordinal scale to interval scale (\href{https://onlinelibrary.wiley.com/doi/10.1155/2014/304213}{Chen & Wang, 2014}).
+#'   The underlying continous distributions
 #'   are estimated analytically for indicators of exogenous variables, and using an ordered
 #'   probit regression for indicators of endogenous variables. Factor scores are used as
 #'   independent variables the ordered probit regressions. Interaction effects between
@@ -166,6 +170,16 @@
 #'   especially when \eqn{G} is small. If \code{FALSE}, the unadjusted CR0 estimator
 #'   is used. Defaults to \code{TRUE}. Only relevant if \code{cluster} is specified.
 #'
+#' @param sampling.weights A variable name in the data frame containing sampling weight information.
+#'   Depending on the sampling.weights.normalization argument, these weights may be rescaled (or not)
+#'   so that their sum equals the number of observations (total or per group)
+#'
+#' @param sampling.weights.normalization If \code{"none"}, the sampling weights (if provided) will not be
+#'   transformed. If \code{"total"}, the sampling weights are normalized by dividing by the total sum
+#'   of the weights, and multiplying again by the total sample size. If \code{"group"}, the sampling
+#'   weights are normalized per group: by dividing by the sum of the weights (in each group), and
+#'   multiplying again by the group size. The default is \code{"total"}.
+#'
 #' @param rcs Should latent variable indicators be replaced with reliability-corrected
 #'   single item indicators instead? See \code{\link{relcorr_single_item}}.
 #'
@@ -272,6 +286,7 @@
 #' }
 modsem_da <- function(model.syntax = NULL,
                       data = NULL,
+                      group = NULL,
                       method = "lms",
                       verbose = NULL,
                       optimize = NULL,
@@ -309,6 +324,8 @@ modsem_da <- function(model.syntax = NULL,
                       ordered.probit.correction = FALSE,
                       cluster = NULL,
                       cr1s = FALSE,
+                      sampling.weights = NULL,
+                      sampling.weights.normalization = NULL,
                       rcs = FALSE,
                       rcs.choose = NULL,
                       rcs.scale.corrected = TRUE,
@@ -366,6 +383,7 @@ modsem_da <- function(model.syntax = NULL,
        ordered             = ordered,
        probit.correction   = ordered.probit.correction,
        cluster             = cluster,
+       group               = group,
        cr1s                = cr1s,
        rcs                 = rcs,
        rcs.choose          = rcs.choose,
@@ -390,6 +408,7 @@ modsem_da <- function(model.syntax = NULL,
     corrected <- relcorr_single_item(
       syntax          = model.syntax,
       data            = data,
+      group           = group,
       choose          = rcs.choose,
       scale.corrected = rcs.scale.corrected,
       warn.lav        = FALSE
@@ -408,60 +427,72 @@ modsem_da <- function(model.syntax = NULL,
     getMethodSettingsDA(method,
       args =
         list(
-          verbose            = verbose,
-          optimize           = optimize,
-          nodes              = nodes,
-          convergence.abs    = convergence.abs,
-          convergence.rel    = convergence.rel,
-          optimizer          = optimizer,
-          center.data        = center.data,
-          standardize.data   = standardize.data,
-          standardize.out    = standardize.out,
-          standardize        = standardize,
-          mean.observed      = mean.observed,
-          double             = double,
-          calc.se            = calc.se,
-          FIM                = FIM,
-          EFIM.S             = EFIM.S,
-          OFIM.hessian       = OFIM.hessian,
-          EFIM.parametric    = EFIM.parametric,
-          robust.se          = robust.se,
-          R.max              = R.max,
-          max.iter           = max.iter,
-          max.step           = max.step,
-          epsilon            = epsilon,
-          quad.range         = quad.range,
-          adaptive.quad      = adaptive.quad,
-          adaptive.frequency = adaptive.frequency,
-          adaptive.quad.tol  = adaptive.quad.tol,
-          n.threads          = n.threads,
-          algorithm          = algorithm,
-          em.control         = em.control,
-          missing            = missing,
-          orthogonal.x       = orthogonal.x,
-          orthogonal.y       = orthogonal.y,
-          auto.fix.first     = auto.fix.first,
-          auto.fix.single    = auto.fix.single,
-          auto.split.syntax  = auto.split.syntax,
-          cr1s               = cr1s
+          verbose                        = verbose,
+          optimize                       = optimize,
+          nodes                          = nodes,
+          convergence.abs                = convergence.abs,
+          convergence.rel                = convergence.rel,
+          optimizer                      = optimizer,
+          center.data                    = center.data,
+          standardize.data               = standardize.data,
+          standardize.out                = standardize.out,
+          standardize                    = standardize,
+          mean.observed                  = mean.observed,
+          double                         = double,
+          calc.se                        = calc.se,
+          FIM                            = FIM,
+          EFIM.S                         = EFIM.S,
+          OFIM.hessian                   = OFIM.hessian,
+          EFIM.parametric                = EFIM.parametric,
+          robust.se                      = robust.se,
+          R.max                          = R.max,
+          max.iter                       = max.iter,
+          max.step                       = max.step,
+          epsilon                        = epsilon,
+          quad.range                     = quad.range,
+          adaptive.quad                  = adaptive.quad,
+          adaptive.frequency             = adaptive.frequency,
+          adaptive.quad.tol              = adaptive.quad.tol,
+          n.threads                      = n.threads,
+          algorithm                      = algorithm,
+          em.control                     = em.control,
+          missing                        = missing,
+          orthogonal.x                   = orthogonal.x,
+          orthogonal.y                   = orthogonal.y,
+          auto.fix.first                 = auto.fix.first,
+          auto.fix.single                = auto.fix.single,
+          auto.split.syntax              = auto.split.syntax,
+          cr1s                           = cr1s,
+          group                          = group,
+          sampling.weights               = sampling.weights,
+          sampling.weights.normalization = sampling.weights.normalization
         )
     )
 
+  cont.cols <- setdiff(colnames(data), c(cluster, group))
+
+  if (args$center.data)
+    data[cont.cols] <- lapply(data[cont.cols], FUN = centerIfNumeric, scaleFactor = FALSE)
+
+  if (args$standardize.data)
+    data[cont.cols] <- lapply(data[cont.cols], FUN = scaleIfNumeric, scaleFactor = FALSE)
+
+  group.info <- parseModelArgumentsByGroupDA(
+    model.syntax       = model.syntax,
+    cov.syntax         = cov.syntax,
+    data               = data,
+    group              = group,
+    auto.split.syntax  = args$auto.split.syntax,
+    sampling.weights   = sampling.weights,
+    sampling.weights.normalization = args$sampling.weights.normalization
+  )
+
   stopif(!method %in% c("lms", "qml"), "Method must be either 'lms' or 'qml'")
 
-  if (args$center.data) {
-    data <- lapplyDf(data, FUN = function(x) x - mean(x, na.rm = TRUE))
-  }
-
-  if (args$standardize.data) {
-    data <- lapplyDf(data, FUN = scaleIfNumeric, scaleFactor = FALSE)
-  }
-
-  model <- specifyModelDA(model.syntax,
-    data               = data,
+  model <- specifyModelDA(
+    group.info         = group.info,
     method             = method,
     m                  = args$nodes,
-    cov.syntax         = cov.syntax,
     mean.observed      = args$mean.observed,
     double             = args$double,
     quad.range         = args$quad.range,
@@ -472,14 +503,22 @@ modsem_da <- function(model.syntax = NULL,
     orthogonal.y       = args$orthogonal.y,
     auto.fix.first     = args$auto.fix.first,
     auto.fix.single    = args$auto.fix.single,
-    auto.split.syntax  = args$auto.split.syntax,
-    cluster            = cluster
+    cluster            = cluster,
+    sampling.weights   = sampling.weights
   )
 
   if (args$optimize) {
     model <- tryCatch({
       .optimize <- purrr::quietly(optimizeStartingParamsDA)
-      result    <- .optimize(model, args = args, engine = "sam")
+      # .optimize <- \(...) list(result = optimizeStartingParamsDA(...))
+      result    <- .optimize(
+        model            = model,
+        args             = args,
+        group            = group,
+        sampling.weights = sampling.weights,
+        engine           = "sam"
+      )
+
       warnings  <- result$warnings
 
       if (length(warnings)) {
@@ -567,8 +606,8 @@ modsem_da <- function(model.syntax = NULL,
   est$expected.matrices <- tryCatch(
     calcExpectedMatricesDA(
       parTable = est$parTable,
-      xis  = getXisModelDA(model), # taking both the main model and cov model into account
-      etas = getEtasModelDA(model)  # taking both the main model and cov model into account
+      xis  = getXisModelDA(model$models[[1L]]), # taking both the main model and cov model into account
+      etas = getEtasModelDA(model$models[[1L]])  # taking both the main model and cov model into account
     ),
     error = function(e) {
       warning2("Failed to calculate expected matrices: ", e$message)
