@@ -247,7 +247,8 @@ modsem_pi <- function(model.syntax = NULL,
     return(est)
   }
 
-  if (!is.data.frame(data)) data <- as.data.frame(data)
+  if (!is.data.frame(data))
+    data <- as.data.frame(data)
 
   if (rcs) { # use reliability-correct single items?
     if (!is.null(rcs.choose))
@@ -256,6 +257,7 @@ modsem_pi <- function(model.syntax = NULL,
     corrected <- relcorr_single_item(
       syntax          = model.syntax,
       data            = data,
+      group           = group,
       choose          = rcs.choose,
       scale.corrected = rcs.scale.corrected,
       warn.lav        = FALSE
@@ -305,13 +307,13 @@ modsem_pi <- function(model.syntax = NULL,
     data <- data[completeCases, ]
   }
 
-  if (standardize.data || method %in% auto.scale) {
-    data <- lapplyDf(data, FUN = scaleIfNumeric, scaleFactor = FALSE)
-  }
+  cont.cols <- setdiff(colnames(data), c(cluster, group))
 
-  if (center.data || method %in% auto.center) {
-    data <- lapplyDf(data, FUN = function(x) x - mean(x, na.rm = TRUE))
-  }
+  if (center.data)
+    data[cont.cols] <- lapply(data[cont.cols], FUN = centerIfNumeric, scaleFactor = FALSE)
+
+  if (standardize.data)
+    data[cont.cols] <- lapply(data[cont.cols], FUN = scaleIfNumeric, scaleFactor = FALSE)
 
   prodInds <-
     createProdInds(modelSpec,
@@ -339,7 +341,7 @@ modsem_pi <- function(model.syntax = NULL,
   if (rcs && rcs.res.cov.xz && method != "ca") { # Constrained Approach Should handle this it on its own...
 
     elemsxz   <- modelSpec$elementsInProdNames
-    crossResCov <- simulateCrosssimulateCrossResCovRCS(
+    crossResCov <- simulateCrossResCovRCS(
       corrected = corrected,
       elemsInIntTerms = elemsxz,
       mc.reps = rcs.mc.reps,
