@@ -1,9 +1,9 @@
 # Calculate weights and node points for mixture functions via Gauss-Hermite
 # quadrature as defined in Klein & Moosbrugger (2000). Also stores information
 # regarding adaptive quadrature.
-quadrature <- function(m, k, 
-                       adaptive = FALSE, 
-                       quad.range = Inf, 
+quadrature <- function(m, k,
+                       adaptive = FALSE,
+                       quad.range = Inf,
                        adaptive.frequency = 3,
                        ...) {
   if (quad.range < 0) {
@@ -16,12 +16,12 @@ quadrature <- function(m, k,
 
   if (k == 0 || m == 0) {
     return(list(
-      n = matrix(0), 
-      w = 1, 
-      k = 0, 
+      n = matrix(0),
+      w = 1,
+      k = 0,
       m = m,
-      a = a, 
-      b = b, 
+      a = a,
+      b = b,
       adaptive = FALSE,
       quad.range = quad.range,
       adaptive.frequency = adaptive.frequency
@@ -43,10 +43,10 @@ quadrature <- function(m, k,
   list(
     n = nodes,
     w = weights,
-    k = k, 
+    k = k,
     m = m,
-    a = a, 
-    b = b, 
+    a = a,
+    b = b,
     quad.range = quad.range,
     adaptive = adaptive,
     adaptive.frequency = adaptive.frequency
@@ -54,10 +54,10 @@ quadrature <- function(m, k,
 }
 
 
-adaptiveGaussQuadrature <- function(fun, 
+adaptiveGaussQuadrature <- function(fun,
                                     collapse = \(x) x, # function to collapse the results, if 'relevant'
-                                    a = -7, 
-                                    b = 7, 
+                                    a = -7,
+                                    b = 7,
                                     m = 32,
                                     m.ceil = m + m / 2,
                                     k = 1,
@@ -66,14 +66,18 @@ adaptiveGaussQuadrature <- function(fun,
                                     tol = 1e-12,
                                     mdiff.tol = 2,
                                     ...) {
-  if (k == 0 || m == 0) return(list(n = matrix(0), w = 1, f = NA, m = 1, k = 1))
-  
+  if (k == 0 || m == 0)
+    return(list(n = matrix(0), w = 1, f = NA, m = 1, k = 1))
+
+  stopif(tol >= 1 || tol < 0,
+         "`adaptive.quad.tol` must be in the boundary `[0, 1)`")
+
   if (k <= 1) {
     out <- adaptiveGaussQuadratureK(
       fun = fun, collapse = collapse,
       a = a, b = b, m = m, m.ceil = m.ceil,
       k = 1, K = k, iter = 1, iter.max = iter.max,
-      node.max = node.max, tol = tol, 
+      node.max = node.max, tol = tol,
       mdiff.tol = mdiff.tol, ...
     )
 
@@ -93,7 +97,7 @@ adaptiveGaussQuadrature <- function(fun,
       fun = fun, collapse = collapse,
       a = a[i], b = b[i], m = m, m.ceil = m.ceil[i],
       k = i, K = k, iter = 1, iter.max = iter.max,
-      node.max = node.max, tol = tol, 
+      node.max = node.max, tol = tol,
       mdiff.tol = mdiff.tol, ...
     )
 
@@ -115,10 +119,10 @@ adaptiveGaussQuadrature <- function(fun,
 }
 
 
-adaptiveGaussQuadratureK <- function(fun, 
+adaptiveGaussQuadratureK <- function(fun,
                                      collapse = \(x) x, # function to collapse the results, if 'relevant'
-                                     a = -7, 
-                                     b = 7, 
+                                     a = -7,
+                                     b = 7,
                                      m = 32,
                                      m.ceil = m + m / 2,
                                      k = 1,
@@ -129,7 +133,11 @@ adaptiveGaussQuadratureK <- function(fun,
                                      tol = 1e-12,
                                      mdiff.tol = 2,
                                      ...) {
-  if (k == 0 || m == 0) return(list(n = matrix(0), w = 1, f = NA, m = 1, k = 1))
+  if (k == 0 || m == 0)
+    return(list(n = matrix(0), w = 1, f = NA, m = 1, k = 1))
+
+  if (is.null(m.ceil) || is.na(m.ceil) || m.ceil <= 0)
+    m.ceil <- round(estMForNodesInRange(m, a = -5, b = 5))
 
   quad  <- quadrature(m = m.ceil, k = 1)
 
@@ -137,20 +145,20 @@ adaptiveGaussQuadratureK <- function(fun,
     quadn <- matrix(0, nrow = m.ceil, ncol = K)
     quadn[,k] <- quad$n
   } else quadn <- quad$n
-  
+
   quadf <- fun(quadn, ...)
   quadw <- matrix(quad$w, ncol = length(quad$w), nrow = NROW(quadf), byrow = TRUE)
 
   integral.full <- collapse(quadf * quadw)
 
-  zeroInfoNodes <- apply(quadw * quadf, MARGIN=2, FUN = \(x) sum(x) <= .Machine$double.xmin * 2)
+  zeroInfoNodes <- apply(quadw * quadf, MARGIN=2, FUN = \(x) sum(x) <= .Machine$double.xmin)
   nodesOutside  <- apply(quadn, MARGIN = 1, FUN = \(x) any(x < a | x > b))
   isValidNode   <- !(zeroInfoNodes | nodesOutside)
 
   quadn <- quadn[isValidNode, , drop = FALSE]
   quadf <- quadf[, isValidNode, drop = FALSE]
   quadw <- quadw[, isValidNode, drop = FALSE]
-  
+
   # vector of full-integral for thresholding
   I.full <- integral.full
 
@@ -172,15 +180,15 @@ adaptiveGaussQuadratureK <- function(fun,
   contrib.rank <- order(abs(contributions))
   cumulative   <- cumsum(contributions[contrib.rank])
   is.removable <- abs(cumulative) < tol * abs(I.full)
-  
+
   # reverse ordering of is.removable
   is.removable <- is.removable[order(contrib.rank)]
   removable    <- which(is.removable)
 
   warnif(sum(contributions[removable]) > tol * abs(I.full),
          "Something went wrong when pruning nodes:\n",
-         "More information than expected was lost!\n")
-  
+         "More information than expected was lost!\n", .newline = TRUE)
+
   stopif(length(removable) >= NROW(quadn), "Cannot remove all nodes!")
 
   if (length(removable) > 0) {
@@ -192,7 +200,7 @@ adaptiveGaussQuadratureK <- function(fun,
     I.cur  <- I.cur - sum(contributions[removable])
   }
 
-  lower  <- min(quadn)
+  lower <- min(quadn)
   upper <- max(quadn)
 
   diff.m <- NROW(quadn) - m
@@ -212,14 +220,14 @@ adaptiveGaussQuadratureK <- function(fun,
   if (!converged && OKIter && OKNextCeil) {
 
     return(adaptiveGaussQuadratureK(
-      fun = fun, 
-      collapse = collapse, 
-      a = a, 
-      b = b, 
-      k = k, 
+      fun = fun,
+      collapse = collapse,
+      a = a,
+      b = b,
+      k = k,
       K = K,
       m = m,
-      m.ceil = new.ceil, 
+      m.ceil = new.ceil,
       iter = iter + 1,
       tol = tol,
       iter.max = iter.max,
@@ -229,20 +237,23 @@ adaptiveGaussQuadratureK <- function(fun,
     ))
   }
 
-  warnif(iter >= iter.max, 
-         "Maximum number of iterations reached when calculating adaptive quadrature!\n",
-         "Try lowering the number of nodes, or increase `quad.range` argument!")
+  warnif(iter >= iter.max,
+         "Max iterations reached fitting quasi-adaptive quadrature...\n",
+         sprintf("Iter %d, total: %d, target: %d, kept: %d, discarded: %d",
+                 iter, m.ceil, m, NROW(quadn), m.ceil - NROW(quadn)),
+         .newline = TRUE)
+
 
   # only need to calculate this before returning the final version
   integral.reduced <- collapse(quadf * quadw)
   error <- integral.reduced - integral.full
 
-  list(n = quadn, 
-       w = quadw[1, , drop = TRUE], 
+  list(n = quadn,
+       w = quadw[1, , drop = TRUE],
        W = quadw,
-       F = quadf, 
-       k = k, 
-       m = nrow(quadn) ^ (1 / k), 
+       F = quadf,
+       k = k,
+       m = nrow(quadn) ^ (1 / k),
 
        m.ceil   = m.ceil,
        iter     = iter,
