@@ -12,7 +12,7 @@ m1 <- '
 
 lms1 <- modsem(m1, oneInt, method = "lms", adaptive.quad=TRUE)
 
-tpb <- ' 
+tpb <- '
 # Outer Model (Based on Hagger et al., 2007)
   ATT =~ att1 + att2 + att3 + att4 + att5
   SN =~ sn1 + sn2
@@ -22,8 +22,8 @@ tpb <- '
 
 # Inner Model (Based on Steinmetz et al., 2011)
   INT ~ ATT + SN + PBC
-  BEH ~ INT + PBC 
-  BEH ~ INT:PBC  
+  BEH ~ INT + PBC
+  BEH ~ INT:PBC
 '
 
 lms2 <- modsem(tpb, TPB, method = "lms", nodes = 32, adaptive.quad=TRUE)
@@ -44,17 +44,17 @@ tpb_uk <- "
   BEH ~ INT:PBC
 "
 
-lms3 <- modsem(tpb_uk, data = TPB_UK, "lms", 
-               nodes=32, FIM="observed",
+lms3 <- modsem(tpb_uk, data = TPB_UK, "lms",
+               nodes=32, FIM="observed", robust.se = TRUE,
                adaptive.quad=TRUE, algorithm ="EMA")
-summary(lms3)
+print(summary(lms3))
 #> Regressions:
 #>                   Estimate  Std.Error  z.value  P(>|z|)
-#>    INT ~         
+#>    INT ~
 #>      PBC             1.037      0.036    28.46    0.000
 #>      ATT            -0.060      0.030    -2.04    0.041
 #>      SN              0.051      0.033     1.55    0.120
-#>    BEH ~         
+#>    BEH ~
 #>      PBC             0.398      0.052     7.63    0.000
 #>      INT             0.594      0.049    12.24    0.000
 #>      PBC:INT         0.141      0.008    17.66    0.000
@@ -62,11 +62,11 @@ summary(lms3)
 # Compared with Mplus
 #> Regressions:
 #>                    Estimate  Std.Error  z.value  Pr(>|z|)
-#>   INT ~   
+#>   INT ~
 #>     ATT              -0.053      0.031    -1.71     0.089
 #>     SN               -0.065      0.024    -2.71     0.008
 #>     PBC               1.090      0.036    30.28     0.000
-#>   BEH ~   
+#>   BEH ~
 #>     PBC               0.405      0.052     7.79     0.000
 #>     INT               0.588      0.048    12.25     0.000
 #>     INT:PBC           0.141      0.008    17.62     0.000
@@ -79,10 +79,10 @@ SC =~ academic1 + academic2 + academic3 + academic4 + academic5 + academic6
 CAREER ~ ENJ + SC + ENJ:ENJ + SC:SC + ENJ:SC
 '
 
-# For such a small number of nodes it doesn't really matter if you use an 
+# For such a small number of nodes it doesn't really matter if you use an
 # adaptive quadrature, as all the nodes bring some value
 # no warning for low number of nodes here, when using adaptive quadrature
-lms4 <- modsem(nlsem, data = jordan, method = "lms", 
+lms4 <- modsem(nlsem, data = jordan, method = "lms",
                adaptive.quad=TRUE, OFIM.hessian = FALSE,
                nodes = 15, mean.observed = FALSE)
 summary(lms4)
